/**************************************************************************
*  Copyright (c) 2016 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  05.01.2016  mifi  First Version, tested with TinyCTS.
*  03.04.2016  mifi  Renamed from talmem.c to talmem_tlsf.c.
**************************************************************************/
#define __TALMEM_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include <stdint.h>
#include <string.h>
#include "tal.h"
#include "tlsf.h"


/*
 * Configure external memory for the HEAP
 */
#define TAL_HEAP_MEM1_START   __SDRAM_segment_used_end__
#define TAL_HEAP_MEM1_END     __SDRAM_segment_end__

extern uint32_t TAL_HEAP_MEM1_START;
extern uint32_t TAL_HEAP_MEM1_END;

extern uint32_t TAL_HEAP_MEM2_START;
extern uint32_t TAL_HEAP_MEM2_END;  

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static char   *pMemPoolRootStart = NULL;
static uint32_t MemPoolRootSize  = 0;

static char   *pMemPool2Start = NULL;
static uint32_t MemPool2Size  = 0;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  tal_BoardMemInit                                                     */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void tal_BoardMemInit (void)
{
   uint32_t Addr;
   uint32_t Size;
   
   /*
    * Two Levels Segregate Fit memory allocator (TLSF) initialization.
    */
    
   /* 
    * Init first memory area 
    */    
    
   /* Get memory pointer */ 
   Addr = (uint32_t)&TAL_HEAP_MEM1_START;
   Size = ((uint32_t)(&TAL_HEAP_MEM1_END) - (uint32_t)(&TAL_HEAP_MEM1_START)) - 1;
   
   /* Align to 16-byte boundary */
   Addr  = (Addr + 15) & ~15;
   Size -= 16;
   pMemPoolRootStart = (char*)Addr;
   MemPoolRootSize   = Size;
   
   /* Clear memory */
   memset(pMemPoolRootStart, 0x00, 16); 
   
   /* Init first memory area */
   init_memory_pool(MemPoolRootSize, pMemPoolRootStart);

#if defined(TAL_HEAP_MEM2_START) && defined(TAL_HEAP_MEM2_END)
   /*
    * Init second memory area
    */
    
   /* Get memory pointer */ 
   Addr = (uint32_t)&TAL_HEAP_MEM2_START;
   Size = ((uint32_t)(&TAL_HEAP_MEM2_END) - (uint32_t)(&TAL_HEAP_MEM2_START)) - 1;

   /* Align to 16-byte boundary */
   Addr  = (Addr + 15) & ~15;
   Size -= 16;
   
   pMemPool2Start = (char*)Addr;
   MemPool2Size   = Size;
      
   /* Add new memory area if available */
   if ((pMemPool2Start != NULL) && (MemPool2Size != 0))
   {
      /* Clear memory */
      memset(pMemPool2Start, 0x00, 16); 
   
      /* Add new memora area */
      add_new_area(pMemPool2Start, MemPool2Size, pMemPoolRootStart);
   } 
#endif     
   
} /* tal_BoardMemInit */


void *calloc (size_t nobj, size_t size)
{
   return( tlsf_calloc(nobj, size) );
} /* calloc */

void *malloc (size_t size)
{
   return( tlsf_malloc(size) );
} /* malloc */

void free (void *addr)
{
   tlsf_free(addr);
} /* free */

void *realloc (void *p, size_t size)
{
   return( tlsf_realloc(p, size) );
} /* realloc */

/*** EOF ***/
