/**************************************************************************
*  This file is part of the TAL project (Tiny Abstraction Layer)
*
*  Copyright (c) 2016 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  21.02.2016  mifi  First Version, tested with an Embedded Artists 
*                    LPC1788 Developer's Kit.
**************************************************************************/
#if !defined(__TALCPU_CAN_H__)
#define __TALCPU_CAN_H__

/**************************************************************************
*  Includes
**************************************************************************/
#include <stdint.h>

/**************************************************************************
*  Global Definitions
**************************************************************************/

/*
 * Filter counts
 */
#define CAN_MAX_FILTER_COUNT  32


/*
 * The LPC1788 CPU supports up to 2 CAN ports.
 */
typedef enum _tal_can_port_
{
   TAL_CAN_PORT_1 = 0,  /* Must be start at 0 */
   TAL_CAN_PORT_2,

   TAL_CAN_PORT_MAX
} TAL_CAN_PORT; 


/*
 * Some statistic information
 */
typedef struct _tal_can_stat_
{
   uint32_t    dRXCount;
   uint32_t    dTXCount;
   uint32_t    dOverunCount;
} TAL_CAN_STAT;
 
/*
 * Hardware information
 */
typedef struct _tal_can_hw_
{
   uint32_t      dBaseAddress;
   uint32_t       FilterArray[CAN_MAX_FILTER_COUNT];
   uint8_t       bFilterAllStd; 
   uint8_t       bFilterAllExt;
   TAL_CAN_STAT   Stat; 
} TAL_CAN_HW;

/**************************************************************************
*  Macro Definitions
**************************************************************************/

/**************************************************************************
*  Functions Definitions
**************************************************************************/

/*
 * Functions still defined in talcan.h
 */

#endif /* !__TALCPU_CAN_H__ */

/*** EOF ***/
